@@ -20,6 +20,9 @@ SPDX-License-Identifier: MIT
 #include "llvm/IR/Constants.h"
 #include "llvm/IR/Function.h"
 #include "llvm/IR/Metadata.h"
+#if VC_INTR_LLVM_VERSION_MAJOR >= 16
+#include <llvm/Support/ModRef.h>
+#endif
 #include "llvm/IR/Module.h"
 #include "llvm/Pass.h"
 #include "llvm/Support/Process.h"
@@ -27,6 +30,7 @@ SPDX-License-Identifier: MIT
 #include "llvmVCWrapper/IR/Attributes.h"
 #include "llvmVCWrapper/IR/DerivedTypes.h"
 #include "llvmVCWrapper/IR/Function.h"
+#include "llvmVCWrapper/IR/Instructions.h"
 
 using namespace llvm;
 using namespace genx;
@@ -54,7 +58,7 @@ private:
   void overrideOptionsWithEnv() {
     auto RewriteSEVOpt = llvm::sys::Process::GetEnv("GENX_REWRITE_SEV");
     if (RewriteSEVOpt)
-      RewriteSingleElementVectors = RewriteSEVOpt.getValue() == "1";
+      RewriteSingleElementVectors = VCINTR::getValue(RewriteSEVOpt) == "1";
   }
 
   bool runOnFunction(Function &F);
@@ -233,6 +237,13 @@ static Instruction *rewriteArgumentUses(Argument &OldArg, Argument &NewArg) {
       M, GenXIntrinsic::genx_address_convert, {OldTy, NewTy});
   ConvFn->addFnAttr(VCFunctionMD::VCFunction);
   auto *Conv = CallInst::Create(ConvFn, {&NewArg});
+#if VC_INTR_LLVM_VERSION_MAJOR >= 16
+  // Modify ReadNone attribute to support llvm16
+  if (ConvFn->getFnAttribute(llvm::Attribute::ReadNone).isValid()) {
+    ConvFn->removeFnAttr(llvm::Attribute::ReadNone);
+    Conv->setMemoryEffects(llvm::MemoryEffects::none());
+  }
+#endif
   OldArg.replaceAllUsesWith(Conv);
   return Conv;
 }
@@ -262,7 +273,7 @@ static SPIRVArgDesc parseArgDesc(StringRef Desc) {
                .Case(ArgDesc::Image3d, SPIRVType::Image3d)
                .Case(ArgDesc::SVM, SPIRVType::Pointer)
                .Case(ArgDesc::Sampler, SPIRVType::Sampler)
-               .Default(None);
+               .Default({});
     }
 
     if (!AccTy) {
@@ -270,7 +281,7 @@ static SPIRVArgDesc parseArgDesc(StringRef Desc) {
                   .Case(ArgDesc::ReadOnly, AccessType::ReadOnly)
                   .Case(ArgDesc::WriteOnly, AccessType::WriteOnly)
                   .Case(ArgDesc::ReadWrite, AccessType::ReadWrite)
-                  .Default(None);
+                  .Default({});
     }
 
     if (Ty && AccTy)
@@ -285,7 +296,7 @@ static SPIRVArgDesc parseArgDesc(StringRef Desc) {
   if (!AccTy)
     AccTy = AccessType::ReadWrite;
 
-  return {Ty.getValue(), AccTy.getValue()};
+  return {VCINTR::getValue(Ty), VCINTR::getValue(AccTy)};
 }
 
 // General arguments can be either pointers or any other types.
@@ -361,7 +372,7 @@ static Optional<ArgKind> extractArgumentKind(const Argument &Arg) {
   const Function *F = Arg.getParent();
   const AttributeList Attrs = F->getAttributes();
   if (!Attrs.hasParamAttr(Arg.getArgNo(), VCFunctionMD::VCArgumentKind))
-    return None;
+    return {};
 
   const Attribute Attr =
       Attrs.getParamAttr(Arg.getArgNo(), VCFunctionMD::VCArgumentKind);
@@ -386,7 +397,7 @@ static StringRef extractArgumentDesc(const Argument &Arg) {
 static SPIRVArgDesc analyzeKernelArg(const Argument &Arg) {
   if (auto Kind = extractArgumentKind(Arg)) {
     const StringRef Desc = extractArgumentDesc(Arg);
-    return analyzeArgumentAttributes(Kind.getValue(), Desc);
+    return analyzeArgumentAttributes(VCINTR::getValue(Kind), Desc);
   }
 
   return {SPIRVType::None};
@@ -423,7 +434,11 @@ static void rewriteKernelArguments(Function &F) {
 
   Function *NewF = transformKernelSignature(F, ArgDescs);
   F.getParent()->getFunctionList().insert(F.getIterator(), NewF);
+#if VC_INTR_LLVM_VERSION_MAJOR > 15
+  NewF->splice(NewF->begin(), &F);
+#else
   NewF->getBasicBlockList().splice(NewF->begin(), F.getBasicBlockList());
+#endif
 
   Instruction *InsPt = &NewF->getEntryBlock().front();
   for (auto ArgPair : llvm::zip(F.args(), NewF->args())) {
